
#include <pmu-events/pmu-events.h>
#include "util/header.h"
#include "util/pmu.h"
#include <string.h>
#include <stddef.h>

struct compact_pmu_event {
        int offset;
};

struct pmu_table_entry {
        const struct compact_pmu_event *entries;
        uint32_t num_entries;
        struct compact_pmu_event pmu_name;
};

static const char *const big_c_string =
/* offset=0 */ "default_core\000"
/* offset=13 */ "fw_misaligned_load\000firmware\000Misaligned load trap event\000config=0x8000000000000000\000\00000\000Misaligned load trap\000"
/* offset=119 */ "fw_misaligned_store\000firmware\000Misaligned store trap event\000config=0x8000000000000001\000\00000\000Misaligned store trap\000"
/* offset=228 */ "fw_access_load\000firmware\000Load access trap event\000config=0x8000000000000002\000\00000\000Load access trap\000"
/* offset=322 */ "fw_access_store\000firmware\000Store access trap event\000config=0x8000000000000003\000\00000\000Store access trap\000"
/* offset=419 */ "fw_illegal_insn\000firmware\000Illegal instruction trap event\000config=0x8000000000000004\000\00000\000Illegal instruction trap\000"
/* offset=530 */ "fw_set_timer\000firmware\000Set timer event\000config=0x8000000000000005\000\00000\000Set timer event\000"
/* offset=614 */ "fw_ipi_sent\000firmware\000Sent IPI to other HART event\000config=0x8000000000000006\000\00000\000Sent IPI to other HART event\000"
/* offset=723 */ "fw_ipi_received\000firmware\000Received IPI from other HART event\000config=0x8000000000000007\000\00000\000Received IPI from other HART event\000"
/* offset=848 */ "fw_fence_i_sent\000firmware\000Sent FENCE.I request to other HART event\000config=0x8000000000000008\000\00000\000Sent FENCE.I request to other HART event\000"
/* offset=985 */ "fw_fence_i_received\000firmware\000Received FENCE.I request from other HART event\000config=0x8000000000000009\000\00000\000Received FENCE.I request from other HART event\000"
/* offset=1138 */ "fw_sfence_vma_sent\000firmware\000Sent SFENCE.VMA request to other HART event\000config=0x800000000000000a\000\00000\000Sent SFENCE.VMA request to other HART event\000"
/* offset=1284 */ "fw_sfence_vma_received\000firmware\000Sent SFENCE.VMA with ASID request to other HART event\000config=0x800000000000000c\000\00000\000Sent SFENCE.VMA with ASID request to other HART event\000"
/* offset=1454 */ "fw_sfence_vma_asid_received\000firmware\000Received SFENCE.VMA with ASID request from other HART event\000config=0x800000000000000d\000\00000\000Received SFENCE.VMA with ASID request from other HART event\000"
/* offset=1641 */ "fw_hfence_gvma_sent\000firmware\000Sent HFENCE.GVMA request to other HART event\000config=0x800000000000000e\000\00000\000Sent HFENCE.GVMA request to other HART event\000"
/* offset=1790 */ "fw_hfence_gvma_received\000firmware\000Received HFENCE.GVMA request from other HART event\000config=0x800000000000000f\000\00000\000Received HFENCE.GVMA request from other HART event\000"
/* offset=1955 */ "fw_hfence_gvma_vmid_sent\000firmware\000Sent HFENCE.GVMA with VMID request to other HART event\000config=0x8000000000000010\000\00000\000Sent HFENCE.GVMA with VMID request to other HART event\000"
/* offset=2129 */ "fw_hfence_gvma_vmid_received\000firmware\000Received HFENCE.GVMA with VMID request from other HART event\000config=0x8000000000000011\000\00000\000Received HFENCE.GVMA with VMID request from other HART event\000"
/* offset=2319 */ "fw_hfence_vvma_sent\000firmware\000Sent HFENCE.VVMA request to other HART event\000config=0x8000000000000012\000\00000\000Sent HFENCE.VVMA request to other HART event\000"
/* offset=2468 */ "fw_hfence_vvma_received\000firmware\000Received HFENCE.VVMA request from other HART event\000config=0x8000000000000013\000\00000\000Received HFENCE.VVMA request from other HART event\000"
/* offset=2633 */ "fw_hfence_vvma_asid_sent\000firmware\000Sent HFENCE.VVMA with ASID request to other HART event\000config=0x8000000000000014\000\00000\000Sent HFENCE.VVMA with ASID request to other HART event\000"
/* offset=2807 */ "fw_hfence_vvma_asid_received\000firmware\000Received HFENCE.VVMA with ASID request from other HART event\000config=0x8000000000000015\000\00000\000Received HFENCE.VVMA with ASID request from other HART event\000"
/* offset=2997 */ "exception_taken\000instructions\000Exception taken\000event=0x100\000\00000\000\000"
/* offset=3059 */ "integer_load_retired\000instructions\000Integer load instruction retired\000event=0x200\000\00000\000\000"
/* offset=3143 */ "integer_store_retired\000instructions\000Integer store instruction retired\000event=0x400\000\00000\000\000"
/* offset=3229 */ "atomic_memory_retired\000instructions\000Atomic memory operation retired\000event=0x800\000\00000\000\000"
/* offset=3313 */ "system_instruction_retired\000instructions\000System instruction retired\000event=0x1000\000\00000\000\000"
/* offset=3398 */ "integer_arithmetic_retired\000instructions\000Integer arithmetic instruction retired\000event=0x2000\000\00000\000\000"
/* offset=3495 */ "conditional_branch_retired\000instructions\000Conditional branch retired\000event=0x4000\000\00000\000\000"
/* offset=3580 */ "jal_instruction_retired\000instructions\000JAL instruction retired\000event=0x8000\000\00000\000\000"
/* offset=3659 */ "jalr_instruction_retired\000instructions\000JALR instruction retired\000event=0x10000\000\00000\000\000"
/* offset=3741 */ "integer_multiplication_retired\000instructions\000Integer multiplication instruction retired\000event=0x20000\000\00000\000\000"
/* offset=3847 */ "integer_division_retired\000instructions\000Integer division instruction retired\000event=0x40000\000\00000\000\000"
/* offset=3941 */ "fp_load_retired\000instructions\000Floating-point load instruction retired\000event=0x80000\000\00000\000\000"
/* offset=4029 */ "fp_store_retired\000instructions\000Floating-point store instruction retired\000event=0x100000\000\00000\000\000"
/* offset=4120 */ "fp_addition_retired\000instructions\000Floating-point addition retired\000event=0x200000\000\00000\000\000"
/* offset=4205 */ "fp_multiplication_retired\000instructions\000Floating-point multiplication retired\000event=0x400000\000\00000\000\000"
/* offset=4302 */ "fp_fusedmadd_retired\000instructions\000Floating-point fused multiply-add retired\000event=0x800000\000\00000\000\000"
/* offset=4398 */ "fp_div_sqrt_retired\000instructions\000Floating-point division or square-root retired\000event=0x1000000\000\00000\000\000"
/* offset=4499 */ "other_fp_retired\000instructions\000Other floating-point instruction retired\000event=0x2000000\000\00000\000\000"
/* offset=4591 */ "icache_retired\000memory\000Instruction cache miss\000event=0x102\000\00000\000\000"
/* offset=4653 */ "dcache_miss_mmio_accesses\000memory\000Data cache miss or memory-mapped I/O access\000event=0x202\000\00000\000\000"
/* offset=4747 */ "dcache_writeback\000memory\000Data cache write-back\000event=0x402\000\00000\000\000"
/* offset=4810 */ "inst_tlb_miss\000memory\000Instruction TLB miss\000event=0x802\000\00000\000\000"
/* offset=4869 */ "data_tlb_miss\000memory\000Data TLB miss\000event=0x1002\000\00000\000\000"
/* offset=4922 */ "utlb_miss\000memory\000UTLB miss\000event=0x2002\000\00000\000\000"
/* offset=4967 */ "addressgen_interlock\000microarch\000Address-generation interlock\000event=0x101\000\00000\000\000"
/* offset=5044 */ "longlat_interlock\000microarch\000Long-latency interlock\000event=0x201\000\00000\000\000"
/* offset=5112 */ "csr_read_interlock\000microarch\000CSR read interlock\000event=0x401\000\00000\000\000"
/* offset=5177 */ "icache_itim_busy\000microarch\000Instruction cache/ITIM busy\000event=0x801\000\00000\000\000"
/* offset=5249 */ "dcache_dtim_busy\000microarch\000Data cache/DTIM busy\000event=0x1001\000\00000\000\000"
/* offset=5315 */ "branch_direction_misprediction\000microarch\000Branch direction misprediction\000event=0x2001\000\00000\000\000"
/* offset=5405 */ "branch_target_misprediction\000microarch\000Branch/jump target misprediction\000event=0x4001\000\00000\000\000"
/* offset=5494 */ "pipe_flush_csr_write\000microarch\000Pipeline flush from CSR write\000event=0x8001\000\00000\000\000"
/* offset=5573 */ "pipe_flush_other_event\000microarch\000Pipeline flush from other event\000event=0x10001\000\00000\000\000"
/* offset=5657 */ "integer_multiplication_interlock\000microarch\000Integer multiplication interlock\000event=0x20001\000\00000\000\000"
/* offset=5752 */ "fp_interlock\000microarch\000Floating-point interlock\000event=0x40001\000\00000\000\000"
/* offset=5819 */ "bp_l1_btb_correct\000branch\000L1 BTB Correction\000event=0x8a\000\00000\000\000"
/* offset=5878 */ "bp_l2_btb_correct\000branch\000L2 BTB Correction\000event=0x8b\000\00000\000\000"
/* offset=5937 */ "l3_cache_rd\000cache\000L3 cache access, read\000event=0x40\000\00000\000Attributable Level 3 cache access, read\000"
/* offset=6032 */ "segment_reg_loads.any\000other\000Number of segment register loads\000event=0x6,period=200000,umask=0x80\000\00000\000\000"
/* offset=6133 */ "dispatch_blocked.any\000other\000Memory cluster signals to block micro-op dispatch for any reason\000event=0x9,period=200000,umask=0x20\000\00000\000\000"
/* offset=6265 */ "eist_trans\000other\000Number of Enhanced Intel SpeedStep(R) Technology (EIST) transitions\000event=0x3a,period=200000,umask=0x0\000\00000\000\000"
/* offset=6390 */ "hisi_sccl,ddrc\000"
/* offset=6405 */ "uncore_hisi_ddrc.flux_wcmd\000uncore\000DDRC write commands\000event=0x2\000\00000\000DDRC write commands\000"
/* offset=6493 */ "uncore_cbox\000"
/* offset=6505 */ "unc_cbo_xsnp_response.miss_eviction\000uncore\000A cross-core snoop resulted from L3 Eviction which misses in some processor core\000event=0x22,umask=0x81\000\00000\000A cross-core snoop resulted from L3 Eviction which misses in some processor core\000"
/* offset=6736 */ "event-hyphen\000uncore\000UNC_CBO_HYPHEN\000event=0xe0,umask=0x00\000\00000\000UNC_CBO_HYPHEN\000"
/* offset=6812 */ "event-two-hyph\000uncore\000UNC_CBO_TWO_HYPH\000event=0xc0,umask=0x00\000\00000\000UNC_CBO_TWO_HYPH\000"
/* offset=6894 */ "hisi_sccl,l3c\000"
/* offset=6908 */ "uncore_hisi_l3c.rd_hit_cpipe\000uncore\000Total read hits\000event=0x7\000\00000\000Total read hits\000"
/* offset=6990 */ "uncore_imc_free_running\000"
/* offset=7014 */ "uncore_imc_free_running.cache_miss\000uncore\000Total cache misses\000event=0x12\000\00000\000Total cache misses\000"
/* offset=7109 */ "uncore_imc\000"
/* offset=7120 */ "uncore_imc.cache_hits\000uncore\000Total cache hits\000event=0x34\000\00000\000Total cache hits\000"
/* offset=7198 */ "uncore_sys_ddr_pmu\000"
/* offset=7217 */ "sys_ddr_pmu.write_cycles\000uncore\000ddr write-cycles event\000event=0x2b\000v8\00000\000\000"
/* offset=7290 */ "uncore_sys_ccn_pmu\000"
/* offset=7309 */ "sys_ccn_pmu.read_cycles\000uncore\000ccn read-cycles event\000config=0x2c\0000x01\00000\000\000"
/* offset=7383 */ "uncore_sys_cmn_pmu\000"
/* offset=7402 */ "sys_cmn_pmu.hnf_cache_miss\000uncore\000Counts total cache misses in first lookup result (high priority)\000eventid=0x1,type=0x5\000(434|436|43c|43a).*\00000\000\000"
/* offset=7546 */ "CPI\000\0001 / IPC\000\000\000\000\000\000\000\00000"
/* offset=7568 */ "IPC\000group1\000inst_retired.any / cpu_clk_unhalted.thread\000\000\000\000\000\000\000\00000"
/* offset=7631 */ "Frontend_Bound_SMT\000\000idq_uops_not_delivered.core / (4 * (cpu_clk_unhalted.thread / 2 * (1 + cpu_clk_unhalted.one_thread_active / cpu_clk_unhalted.ref_xclk)))\000\000\000\000\000\000\000\00000"
/* offset=7797 */ "dcache_miss_cpi\000\000l1d\\-loads\\-misses / inst_retired.any\000\000\000\000\000\000\000\00000"
/* offset=7861 */ "icache_miss_cycles\000\000l1i\\-loads\\-misses / inst_retired.any\000\000\000\000\000\000\000\00000"
/* offset=7928 */ "cache_miss_cycles\000group1\000dcache_miss_cpi + icache_miss_cycles\000\000\000\000\000\000\000\00000"
/* offset=7999 */ "DCache_L2_All_Hits\000\000l2_rqsts.demand_data_rd_hit + l2_rqsts.pf_hit + l2_rqsts.rfo_hit\000\000\000\000\000\000\000\00000"
/* offset=8093 */ "DCache_L2_All_Miss\000\000max(l2_rqsts.all_demand_data_rd - l2_rqsts.demand_data_rd_hit, 0) + l2_rqsts.pf_miss + l2_rqsts.rfo_miss\000\000\000\000\000\000\000\00000"
/* offset=8227 */ "DCache_L2_All\000\000DCache_L2_All_Hits + DCache_L2_All_Miss\000\000\000\000\000\000\000\00000"
/* offset=8291 */ "DCache_L2_Hits\000\000d_ratio(DCache_L2_All_Hits, DCache_L2_All)\000\000\000\000\000\000\000\00000"
/* offset=8359 */ "DCache_L2_Misses\000\000d_ratio(DCache_L2_All_Miss, DCache_L2_All)\000\000\000\000\000\000\000\00000"
/* offset=8429 */ "M1\000\000ipc + M2\000\000\000\000\000\000\000\00000"
/* offset=8451 */ "M2\000\000ipc + M1\000\000\000\000\000\000\000\00000"
/* offset=8473 */ "M3\000\0001 / M3\000\000\000\000\000\000\000\00000"
/* offset=8493 */ "L1D_Cache_Fill_BW\000\00064 * l1d.replacement / 1e9 / duration_time\000\000\000\000\000\000\000\00000"
;

static const struct compact_pmu_event pmu_events__sifive_u74_default_core[] = {
{ 4967 }, /* addressgen_interlock\000microarch\000Address-generation interlock\000event=0x101\000\00000\000\000 */
{ 3229 }, /* atomic_memory_retired\000instructions\000Atomic memory operation retired\000event=0x800\000\00000\000\000 */
{ 5315 }, /* branch_direction_misprediction\000microarch\000Branch direction misprediction\000event=0x2001\000\00000\000\000 */
{ 5405 }, /* branch_target_misprediction\000microarch\000Branch/jump target misprediction\000event=0x4001\000\00000\000\000 */
{ 3495 }, /* conditional_branch_retired\000instructions\000Conditional branch retired\000event=0x4000\000\00000\000\000 */
{ 5112 }, /* csr_read_interlock\000microarch\000CSR read interlock\000event=0x401\000\00000\000\000 */
{ 4869 }, /* data_tlb_miss\000memory\000Data TLB miss\000event=0x1002\000\00000\000\000 */
{ 5249 }, /* dcache_dtim_busy\000microarch\000Data cache/DTIM busy\000event=0x1001\000\00000\000\000 */
{ 4653 }, /* dcache_miss_mmio_accesses\000memory\000Data cache miss or memory-mapped I/O access\000event=0x202\000\00000\000\000 */
{ 4747 }, /* dcache_writeback\000memory\000Data cache write-back\000event=0x402\000\00000\000\000 */
{ 2997 }, /* exception_taken\000instructions\000Exception taken\000event=0x100\000\00000\000\000 */
{ 4120 }, /* fp_addition_retired\000instructions\000Floating-point addition retired\000event=0x200000\000\00000\000\000 */
{ 4398 }, /* fp_div_sqrt_retired\000instructions\000Floating-point division or square-root retired\000event=0x1000000\000\00000\000\000 */
{ 4302 }, /* fp_fusedmadd_retired\000instructions\000Floating-point fused multiply-add retired\000event=0x800000\000\00000\000\000 */
{ 5752 }, /* fp_interlock\000microarch\000Floating-point interlock\000event=0x40001\000\00000\000\000 */
{ 3941 }, /* fp_load_retired\000instructions\000Floating-point load instruction retired\000event=0x80000\000\00000\000\000 */
{ 4205 }, /* fp_multiplication_retired\000instructions\000Floating-point multiplication retired\000event=0x400000\000\00000\000\000 */
{ 4029 }, /* fp_store_retired\000instructions\000Floating-point store instruction retired\000event=0x100000\000\00000\000\000 */
{ 228 }, /* fw_access_load\000firmware\000Load access trap event\000config=0x8000000000000002\000\00000\000Load access trap\000 */
{ 322 }, /* fw_access_store\000firmware\000Store access trap event\000config=0x8000000000000003\000\00000\000Store access trap\000 */
{ 985 }, /* fw_fence_i_received\000firmware\000Received FENCE.I request from other HART event\000config=0x8000000000000009\000\00000\000Received FENCE.I request from other HART event\000 */
{ 848 }, /* fw_fence_i_sent\000firmware\000Sent FENCE.I request to other HART event\000config=0x8000000000000008\000\00000\000Sent FENCE.I request to other HART event\000 */
{ 1790 }, /* fw_hfence_gvma_received\000firmware\000Received HFENCE.GVMA request from other HART event\000config=0x800000000000000f\000\00000\000Received HFENCE.GVMA request from other HART event\000 */
{ 1641 }, /* fw_hfence_gvma_sent\000firmware\000Sent HFENCE.GVMA request to other HART event\000config=0x800000000000000e\000\00000\000Sent HFENCE.GVMA request to other HART event\000 */
{ 2129 }, /* fw_hfence_gvma_vmid_received\000firmware\000Received HFENCE.GVMA with VMID request from other HART event\000config=0x8000000000000011\000\00000\000Received HFENCE.GVMA with VMID request from other HART event\000 */
{ 1955 }, /* fw_hfence_gvma_vmid_sent\000firmware\000Sent HFENCE.GVMA with VMID request to other HART event\000config=0x8000000000000010\000\00000\000Sent HFENCE.GVMA with VMID request to other HART event\000 */
{ 2807 }, /* fw_hfence_vvma_asid_received\000firmware\000Received HFENCE.VVMA with ASID request from other HART event\000config=0x8000000000000015\000\00000\000Received HFENCE.VVMA with ASID request from other HART event\000 */
{ 2633 }, /* fw_hfence_vvma_asid_sent\000firmware\000Sent HFENCE.VVMA with ASID request to other HART event\000config=0x8000000000000014\000\00000\000Sent HFENCE.VVMA with ASID request to other HART event\000 */
{ 2468 }, /* fw_hfence_vvma_received\000firmware\000Received HFENCE.VVMA request from other HART event\000config=0x8000000000000013\000\00000\000Received HFENCE.VVMA request from other HART event\000 */
{ 2319 }, /* fw_hfence_vvma_sent\000firmware\000Sent HFENCE.VVMA request to other HART event\000config=0x8000000000000012\000\00000\000Sent HFENCE.VVMA request to other HART event\000 */
{ 419 }, /* fw_illegal_insn\000firmware\000Illegal instruction trap event\000config=0x8000000000000004\000\00000\000Illegal instruction trap\000 */
{ 723 }, /* fw_ipi_received\000firmware\000Received IPI from other HART event\000config=0x8000000000000007\000\00000\000Received IPI from other HART event\000 */
{ 614 }, /* fw_ipi_sent\000firmware\000Sent IPI to other HART event\000config=0x8000000000000006\000\00000\000Sent IPI to other HART event\000 */
{ 13 }, /* fw_misaligned_load\000firmware\000Misaligned load trap event\000config=0x8000000000000000\000\00000\000Misaligned load trap\000 */
{ 119 }, /* fw_misaligned_store\000firmware\000Misaligned store trap event\000config=0x8000000000000001\000\00000\000Misaligned store trap\000 */
{ 530 }, /* fw_set_timer\000firmware\000Set timer event\000config=0x8000000000000005\000\00000\000Set timer event\000 */
{ 1454 }, /* fw_sfence_vma_asid_received\000firmware\000Received SFENCE.VMA with ASID request from other HART event\000config=0x800000000000000d\000\00000\000Received SFENCE.VMA with ASID request from other HART event\000 */
{ 1284 }, /* fw_sfence_vma_received\000firmware\000Sent SFENCE.VMA with ASID request to other HART event\000config=0x800000000000000c\000\00000\000Sent SFENCE.VMA with ASID request to other HART event\000 */
{ 1284 }, /* fw_sfence_vma_received\000firmware\000Sent SFENCE.VMA with ASID request to other HART event\000config=0x800000000000000c\000\00000\000Sent SFENCE.VMA with ASID request to other HART event\000 */
{ 1138 }, /* fw_sfence_vma_sent\000firmware\000Sent SFENCE.VMA request to other HART event\000config=0x800000000000000a\000\00000\000Sent SFENCE.VMA request to other HART event\000 */
{ 5177 }, /* icache_itim_busy\000microarch\000Instruction cache/ITIM busy\000event=0x801\000\00000\000\000 */
{ 4591 }, /* icache_retired\000memory\000Instruction cache miss\000event=0x102\000\00000\000\000 */
{ 4810 }, /* inst_tlb_miss\000memory\000Instruction TLB miss\000event=0x802\000\00000\000\000 */
{ 3398 }, /* integer_arithmetic_retired\000instructions\000Integer arithmetic instruction retired\000event=0x2000\000\00000\000\000 */
{ 3847 }, /* integer_division_retired\000instructions\000Integer division instruction retired\000event=0x40000\000\00000\000\000 */
{ 3059 }, /* integer_load_retired\000instructions\000Integer load instruction retired\000event=0x200\000\00000\000\000 */
{ 5657 }, /* integer_multiplication_interlock\000microarch\000Integer multiplication interlock\000event=0x20001\000\00000\000\000 */
{ 3741 }, /* integer_multiplication_retired\000instructions\000Integer multiplication instruction retired\000event=0x20000\000\00000\000\000 */
{ 3143 }, /* integer_store_retired\000instructions\000Integer store instruction retired\000event=0x400\000\00000\000\000 */
{ 3580 }, /* jal_instruction_retired\000instructions\000JAL instruction retired\000event=0x8000\000\00000\000\000 */
{ 3659 }, /* jalr_instruction_retired\000instructions\000JALR instruction retired\000event=0x10000\000\00000\000\000 */
{ 5044 }, /* longlat_interlock\000microarch\000Long-latency interlock\000event=0x201\000\00000\000\000 */
{ 4499 }, /* other_fp_retired\000instructions\000Other floating-point instruction retired\000event=0x2000000\000\00000\000\000 */
{ 5494 }, /* pipe_flush_csr_write\000microarch\000Pipeline flush from CSR write\000event=0x8001\000\00000\000\000 */
{ 5573 }, /* pipe_flush_other_event\000microarch\000Pipeline flush from other event\000event=0x10001\000\00000\000\000 */
{ 3313 }, /* system_instruction_retired\000instructions\000System instruction retired\000event=0x1000\000\00000\000\000 */
{ 4922 }, /* utlb_miss\000memory\000UTLB miss\000event=0x2002\000\00000\000\000 */

};

const struct pmu_table_entry pmu_events__sifive_u74[] = {
{
     .entries = pmu_events__sifive_u74_default_core,
     .num_entries = ARRAY_SIZE(pmu_events__sifive_u74_default_core),
     .pmu_name = { 0 /* default_core\000 */ },
},
};

static const struct compact_pmu_event pmu_events__test_soc_cpu_default_core[] = {
{ 5819 }, /* bp_l1_btb_correct\000branch\000L1 BTB Correction\000event=0x8a\000\00000\000\000 */
{ 5878 }, /* bp_l2_btb_correct\000branch\000L2 BTB Correction\000event=0x8b\000\00000\000\000 */
{ 6133 }, /* dispatch_blocked.any\000other\000Memory cluster signals to block micro-op dispatch for any reason\000event=0x9,period=200000,umask=0x20\000\00000\000\000 */
{ 6265 }, /* eist_trans\000other\000Number of Enhanced Intel SpeedStep(R) Technology (EIST) transitions\000event=0x3a,period=200000,umask=0x0\000\00000\000\000 */
{ 5937 }, /* l3_cache_rd\000cache\000L3 cache access, read\000event=0x40\000\00000\000Attributable Level 3 cache access, read\000 */
{ 6032 }, /* segment_reg_loads.any\000other\000Number of segment register loads\000event=0x6,period=200000,umask=0x80\000\00000\000\000 */
};
static const struct compact_pmu_event pmu_events__test_soc_cpu_hisi_sccl_ddrc[] = {
{ 6405 }, /* uncore_hisi_ddrc.flux_wcmd\000uncore\000DDRC write commands\000event=0x2\000\00000\000DDRC write commands\000 */
};
static const struct compact_pmu_event pmu_events__test_soc_cpu_hisi_sccl_l3c[] = {
{ 6908 }, /* uncore_hisi_l3c.rd_hit_cpipe\000uncore\000Total read hits\000event=0x7\000\00000\000Total read hits\000 */
};
static const struct compact_pmu_event pmu_events__test_soc_cpu_uncore_cbox[] = {
{ 6736 }, /* event-hyphen\000uncore\000UNC_CBO_HYPHEN\000event=0xe0,umask=0x00\000\00000\000UNC_CBO_HYPHEN\000 */
{ 6812 }, /* event-two-hyph\000uncore\000UNC_CBO_TWO_HYPH\000event=0xc0,umask=0x00\000\00000\000UNC_CBO_TWO_HYPH\000 */
{ 6505 }, /* unc_cbo_xsnp_response.miss_eviction\000uncore\000A cross-core snoop resulted from L3 Eviction which misses in some processor core\000event=0x22,umask=0x81\000\00000\000A cross-core snoop resulted from L3 Eviction which misses in some processor core\000 */
};
static const struct compact_pmu_event pmu_events__test_soc_cpu_uncore_imc[] = {
{ 7120 }, /* uncore_imc.cache_hits\000uncore\000Total cache hits\000event=0x34\000\00000\000Total cache hits\000 */
};
static const struct compact_pmu_event pmu_events__test_soc_cpu_uncore_imc_free_running[] = {
{ 7014 }, /* uncore_imc_free_running.cache_miss\000uncore\000Total cache misses\000event=0x12\000\00000\000Total cache misses\000 */

};

const struct pmu_table_entry pmu_events__test_soc_cpu[] = {
{
     .entries = pmu_events__test_soc_cpu_default_core,
     .num_entries = ARRAY_SIZE(pmu_events__test_soc_cpu_default_core),
     .pmu_name = { 0 /* default_core\000 */ },
},
{
     .entries = pmu_events__test_soc_cpu_hisi_sccl_ddrc,
     .num_entries = ARRAY_SIZE(pmu_events__test_soc_cpu_hisi_sccl_ddrc),
     .pmu_name = { 6390 /* hisi_sccl,ddrc\000 */ },
},
{
     .entries = pmu_events__test_soc_cpu_hisi_sccl_l3c,
     .num_entries = ARRAY_SIZE(pmu_events__test_soc_cpu_hisi_sccl_l3c),
     .pmu_name = { 6894 /* hisi_sccl,l3c\000 */ },
},
{
     .entries = pmu_events__test_soc_cpu_uncore_cbox,
     .num_entries = ARRAY_SIZE(pmu_events__test_soc_cpu_uncore_cbox),
     .pmu_name = { 6493 /* uncore_cbox\000 */ },
},
{
     .entries = pmu_events__test_soc_cpu_uncore_imc,
     .num_entries = ARRAY_SIZE(pmu_events__test_soc_cpu_uncore_imc),
     .pmu_name = { 7109 /* uncore_imc\000 */ },
},
{
     .entries = pmu_events__test_soc_cpu_uncore_imc_free_running,
     .num_entries = ARRAY_SIZE(pmu_events__test_soc_cpu_uncore_imc_free_running),
     .pmu_name = { 6990 /* uncore_imc_free_running\000 */ },
},
};

static const struct compact_pmu_event pmu_metrics__test_soc_cpu_default_core[] = {
{ 7546 }, /* CPI\000\0001 / IPC\000\000\000\000\000\000\000\00000 */
{ 8227 }, /* DCache_L2_All\000\000DCache_L2_All_Hits + DCache_L2_All_Miss\000\000\000\000\000\000\000\00000 */
{ 7999 }, /* DCache_L2_All_Hits\000\000l2_rqsts.demand_data_rd_hit + l2_rqsts.pf_hit + l2_rqsts.rfo_hit\000\000\000\000\000\000\000\00000 */
{ 8093 }, /* DCache_L2_All_Miss\000\000max(l2_rqsts.all_demand_data_rd - l2_rqsts.demand_data_rd_hit, 0) + l2_rqsts.pf_miss + l2_rqsts.rfo_miss\000\000\000\000\000\000\000\00000 */
{ 8291 }, /* DCache_L2_Hits\000\000d_ratio(DCache_L2_All_Hits, DCache_L2_All)\000\000\000\000\000\000\000\00000 */
{ 8359 }, /* DCache_L2_Misses\000\000d_ratio(DCache_L2_All_Miss, DCache_L2_All)\000\000\000\000\000\000\000\00000 */
{ 7631 }, /* Frontend_Bound_SMT\000\000idq_uops_not_delivered.core / (4 * (cpu_clk_unhalted.thread / 2 * (1 + cpu_clk_unhalted.one_thread_active / cpu_clk_unhalted.ref_xclk)))\000\000\000\000\000\000\000\00000 */
{ 7568 }, /* IPC\000group1\000inst_retired.any / cpu_clk_unhalted.thread\000\000\000\000\000\000\000\00000 */
{ 8493 }, /* L1D_Cache_Fill_BW\000\00064 * l1d.replacement / 1e9 / duration_time\000\000\000\000\000\000\000\00000 */
{ 8429 }, /* M1\000\000ipc + M2\000\000\000\000\000\000\000\00000 */
{ 8451 }, /* M2\000\000ipc + M1\000\000\000\000\000\000\000\00000 */
{ 8473 }, /* M3\000\0001 / M3\000\000\000\000\000\000\000\00000 */
{ 7928 }, /* cache_miss_cycles\000group1\000dcache_miss_cpi + icache_miss_cycles\000\000\000\000\000\000\000\00000 */
{ 7797 }, /* dcache_miss_cpi\000\000l1d\\-loads\\-misses / inst_retired.any\000\000\000\000\000\000\000\00000 */
{ 7861 }, /* icache_miss_cycles\000\000l1i\\-loads\\-misses / inst_retired.any\000\000\000\000\000\000\000\00000 */

};

const struct pmu_table_entry pmu_metrics__test_soc_cpu[] = {
{
     .entries = pmu_metrics__test_soc_cpu_default_core,
     .num_entries = ARRAY_SIZE(pmu_metrics__test_soc_cpu_default_core),
     .pmu_name = { 0 /* default_core\000 */ },
},
};

static const struct compact_pmu_event pmu_events__test_soc_sys_uncore_sys_ccn_pmu[] = {
{ 7309 }, /* sys_ccn_pmu.read_cycles\000uncore\000ccn read-cycles event\000config=0x2c\0000x01\00000\000\000 */
};
static const struct compact_pmu_event pmu_events__test_soc_sys_uncore_sys_cmn_pmu[] = {
{ 7402 }, /* sys_cmn_pmu.hnf_cache_miss\000uncore\000Counts total cache misses in first lookup result (high priority)\000eventid=0x1,type=0x5\000(434|436|43c|43a).*\00000\000\000 */
};
static const struct compact_pmu_event pmu_events__test_soc_sys_uncore_sys_ddr_pmu[] = {
{ 7217 }, /* sys_ddr_pmu.write_cycles\000uncore\000ddr write-cycles event\000event=0x2b\000v8\00000\000\000 */

};

const struct pmu_table_entry pmu_events__test_soc_sys[] = {
{
     .entries = pmu_events__test_soc_sys_uncore_sys_ccn_pmu,
     .num_entries = ARRAY_SIZE(pmu_events__test_soc_sys_uncore_sys_ccn_pmu),
     .pmu_name = { 7290 /* uncore_sys_ccn_pmu\000 */ },
},
{
     .entries = pmu_events__test_soc_sys_uncore_sys_cmn_pmu,
     .num_entries = ARRAY_SIZE(pmu_events__test_soc_sys_uncore_sys_cmn_pmu),
     .pmu_name = { 7383 /* uncore_sys_cmn_pmu\000 */ },
},
{
     .entries = pmu_events__test_soc_sys_uncore_sys_ddr_pmu,
     .num_entries = ARRAY_SIZE(pmu_events__test_soc_sys_uncore_sys_ddr_pmu),
     .pmu_name = { 7198 /* uncore_sys_ddr_pmu\000 */ },
},
};


/* Struct used to make the PMU event table implementation opaque to callers. */
struct pmu_events_table {
        const struct pmu_table_entry *pmus;
        uint32_t num_pmus;
};

/* Struct used to make the PMU metric table implementation opaque to callers. */
struct pmu_metrics_table {
        const struct pmu_table_entry *pmus;
        uint32_t num_pmus;
};

/*
 * Map a CPU to its table of PMU events. The CPU is identified by the
 * cpuid field, which is an arch-specific identifier for the CPU.
 * The identifier specified in tools/perf/pmu-events/arch/xxx/mapfile
 * must match the get_cpuid_str() in tools/perf/arch/xxx/util/header.c)
 *
 * The  cpuid can contain any character other than the comma.
 */
struct pmu_events_map {
        const char *arch;
        const char *cpuid;
        struct pmu_events_table event_table;
        struct pmu_metrics_table metric_table;
};

/*
 * Global table mapping each known CPU for the architecture to its
 * table of PMU events.
 */
const struct pmu_events_map pmu_events_map[] = {
{
	.arch = "riscv",
	.cpuid = "0x489-0x8000000000000007-0x[[:xdigit:]]+",
	.event_table = {
		.pmus = pmu_events__sifive_u74,
		.num_pmus = ARRAY_SIZE(pmu_events__sifive_u74)
	},
	.metric_table = {
		.pmus = NULL,
		.num_pmus = 0
	}
},
{
	.arch = "testarch",
	.cpuid = "testcpu",
	.event_table = {
		.pmus = pmu_events__test_soc_cpu,
		.num_pmus = ARRAY_SIZE(pmu_events__test_soc_cpu),
	},
	.metric_table = {
		.pmus = pmu_metrics__test_soc_cpu,
		.num_pmus = ARRAY_SIZE(pmu_metrics__test_soc_cpu),
	}
},
{
	.arch = 0,
	.cpuid = 0,
	.event_table = { 0, 0 },
	.metric_table = { 0, 0 },
}
};

struct pmu_sys_events {
	const char *name;
	struct pmu_events_table event_table;
	struct pmu_metrics_table metric_table;
};

static const struct pmu_sys_events pmu_sys_event_tables[] = {
	{
		.event_table = {
			.pmus = pmu_events__test_soc_sys,
			.num_pmus = ARRAY_SIZE(pmu_events__test_soc_sys)
		},
		.name = "pmu_events__test_soc_sys",
	},
	{
		.event_table = { 0, 0 },
		.metric_table = { 0, 0 },
	},
};

static void decompress_event(int offset, struct pmu_event *pe)
{
	const char *p = &big_c_string[offset];

	pe->name = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->topic = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->desc = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->event = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->compat = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->deprecated = *p - '0';
	p++;
	pe->perpkg = *p - '0';
	p++;
	pe->unit = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->long_desc = (*p == '\0' ? NULL : p);
}

static void decompress_metric(int offset, struct pmu_metric *pm)
{
	const char *p = &big_c_string[offset];

	pm->metric_name = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->metric_group = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->metric_expr = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->metric_threshold = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->desc = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->long_desc = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->unit = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->compat = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->metricgroup_no_group = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->default_metricgroup_name = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->aggr_mode = *p - '0';
	p++;
	pm->event_grouping = *p - '0';
}

static int pmu_events_table__for_each_event_pmu(const struct pmu_events_table *table,
                                                const struct pmu_table_entry *pmu,
                                                pmu_event_iter_fn fn,
                                                void *data)
{
        int ret;
        struct pmu_event pe = {
                .pmu = &big_c_string[pmu->pmu_name.offset],
        };

        for (uint32_t i = 0; i < pmu->num_entries; i++) {
                decompress_event(pmu->entries[i].offset, &pe);
                if (!pe.name)
                        continue;
                ret = fn(&pe, table, data);
                if (ret)
                        return ret;
        }
        return 0;
 }

static int pmu_events_table__find_event_pmu(const struct pmu_events_table *table,
                                            const struct pmu_table_entry *pmu,
                                            const char *name,
                                            pmu_event_iter_fn fn,
                                            void *data)
{
        struct pmu_event pe = {
                .pmu = &big_c_string[pmu->pmu_name.offset],
        };
        int low = 0, high = pmu->num_entries - 1;

        while (low <= high) {
                int cmp, mid = (low + high) / 2;

                decompress_event(pmu->entries[mid].offset, &pe);

                if (!pe.name && !name)
                        goto do_call;

                if (!pe.name && name) {
                        low = mid + 1;
                        continue;
                }
                if (pe.name && !name) {
                        high = mid - 1;
                        continue;
                }

                cmp = strcasecmp(pe.name, name);
                if (cmp < 0) {
                        low = mid + 1;
                        continue;
                }
                if (cmp > 0) {
                        high = mid - 1;
                        continue;
                }
  do_call:
                return fn ? fn(&pe, table, data) : 0;
        }
        return -1000;
}

int pmu_events_table__for_each_event(const struct pmu_events_table *table,
                                    struct perf_pmu *pmu,
                                    pmu_event_iter_fn fn,
                                    void *data)
{
        for (size_t i = 0; i < table->num_pmus; i++) {
                const struct pmu_table_entry *table_pmu = &table->pmus[i];
                const char *pmu_name = &big_c_string[table_pmu->pmu_name.offset];
                int ret;

                if (pmu && !pmu__name_match(pmu, pmu_name))
                        continue;

                ret = pmu_events_table__for_each_event_pmu(table, table_pmu, fn, data);
                if (pmu || ret)
                        return ret;
        }
        return 0;
}

int pmu_events_table__find_event(const struct pmu_events_table *table,
                                 struct perf_pmu *pmu,
                                 const char *name,
                                 pmu_event_iter_fn fn,
                                 void *data)
{
        for (size_t i = 0; i < table->num_pmus; i++) {
                const struct pmu_table_entry *table_pmu = &table->pmus[i];
                const char *pmu_name = &big_c_string[table_pmu->pmu_name.offset];
                int ret;

                if (!pmu__name_match(pmu, pmu_name))
                        continue;

                ret = pmu_events_table__find_event_pmu(table, table_pmu, name, fn, data);
                if (ret != -1000)
                        return ret;
        }
        return -1000;
}

size_t pmu_events_table__num_events(const struct pmu_events_table *table,
                                    struct perf_pmu *pmu)
{
        size_t count = 0;

        for (size_t i = 0; i < table->num_pmus; i++) {
                const struct pmu_table_entry *table_pmu = &table->pmus[i];
                const char *pmu_name = &big_c_string[table_pmu->pmu_name.offset];

                if (pmu__name_match(pmu, pmu_name))
                        count += table_pmu->num_entries;
        }
        return count;
}

static int pmu_metrics_table__for_each_metric_pmu(const struct pmu_metrics_table *table,
                                                const struct pmu_table_entry *pmu,
                                                pmu_metric_iter_fn fn,
                                                void *data)
{
        int ret;
        struct pmu_metric pm = {
                .pmu = &big_c_string[pmu->pmu_name.offset],
        };

        for (uint32_t i = 0; i < pmu->num_entries; i++) {
                decompress_metric(pmu->entries[i].offset, &pm);
                if (!pm.metric_expr)
                        continue;
                ret = fn(&pm, table, data);
                if (ret)
                        return ret;
        }
        return 0;
}

int pmu_metrics_table__for_each_metric(const struct pmu_metrics_table *table,
                                     pmu_metric_iter_fn fn,
                                     void *data)
{
        for (size_t i = 0; i < table->num_pmus; i++) {
                int ret = pmu_metrics_table__for_each_metric_pmu(table, &table->pmus[i],
                                                                 fn, data);

                if (ret)
                        return ret;
        }
        return 0;
}

static const struct pmu_events_map *map_for_pmu(struct perf_pmu *pmu)
{
        static struct {
                const struct pmu_events_map *map;
                struct perf_pmu *pmu;
        } last_result;
        static struct {
                const struct pmu_events_map *map;
                char *cpuid;
        } last_map_search;
        static bool has_last_result, has_last_map_search;
        const struct pmu_events_map *map = NULL;
        char *cpuid = NULL;
        size_t i;

        if (has_last_result && last_result.pmu == pmu)
                return last_result.map;

        cpuid = perf_pmu__getcpuid(pmu);

        /*
         * On some platforms which uses cpus map, cpuid can be NULL for
         * PMUs other than CORE PMUs.
         */
        if (!cpuid)
                goto out_update_last_result;

        if (has_last_map_search && !strcmp(last_map_search.cpuid, cpuid)) {
                map = last_map_search.map;
                free(cpuid);
        } else {
                i = 0;
                for (;;) {
                        map = &pmu_events_map[i++];

                        if (!map->arch) {
                                map = NULL;
                                break;
                        }

                        if (!strcmp_cpuid_str(map->cpuid, cpuid))
                                break;
               }
               free(last_map_search.cpuid);
               last_map_search.cpuid = cpuid;
               last_map_search.map = map;
               has_last_map_search = true;
        }
out_update_last_result:
        last_result.pmu = pmu;
        last_result.map = map;
        has_last_result = true;
        return map;
}

const struct pmu_events_table *perf_pmu__find_events_table(struct perf_pmu *pmu)
{
        const struct pmu_events_map *map = map_for_pmu(pmu);

        if (!map)
                return NULL;

        if (!pmu)
                return &map->event_table;

        for (size_t i = 0; i < map->event_table.num_pmus; i++) {
                const struct pmu_table_entry *table_pmu = &map->event_table.pmus[i];
                const char *pmu_name = &big_c_string[table_pmu->pmu_name.offset];

                if (pmu__name_match(pmu, pmu_name))
                         return &map->event_table;
        }
        return NULL;
}

const struct pmu_metrics_table *perf_pmu__find_metrics_table(struct perf_pmu *pmu)
{
        const struct pmu_events_map *map = map_for_pmu(pmu);

        if (!map)
                return NULL;

        if (!pmu)
                return &map->metric_table;

        for (size_t i = 0; i < map->metric_table.num_pmus; i++) {
                const struct pmu_table_entry *table_pmu = &map->metric_table.pmus[i];
                const char *pmu_name = &big_c_string[table_pmu->pmu_name.offset];

                if (pmu__name_match(pmu, pmu_name))
                           return &map->metric_table;
        }
        return NULL;
}

const struct pmu_events_table *find_core_events_table(const char *arch, const char *cpuid)
{
        for (const struct pmu_events_map *tables = &pmu_events_map[0];
             tables->arch;
             tables++) {
                if (!strcmp(tables->arch, arch) && !strcmp_cpuid_str(tables->cpuid, cpuid))
                        return &tables->event_table;
        }
        return NULL;
}

const struct pmu_metrics_table *find_core_metrics_table(const char *arch, const char *cpuid)
{
        for (const struct pmu_events_map *tables = &pmu_events_map[0];
             tables->arch;
             tables++) {
                if (!strcmp(tables->arch, arch) && !strcmp_cpuid_str(tables->cpuid, cpuid))
                        return &tables->metric_table;
        }
        return NULL;
}

int pmu_for_each_core_event(pmu_event_iter_fn fn, void *data)
{
        for (const struct pmu_events_map *tables = &pmu_events_map[0];
             tables->arch;
             tables++) {
                int ret = pmu_events_table__for_each_event(&tables->event_table,
                                                           /*pmu=*/ NULL, fn, data);

                if (ret)
                        return ret;
        }
        return 0;
}

int pmu_for_each_core_metric(pmu_metric_iter_fn fn, void *data)
{
        for (const struct pmu_events_map *tables = &pmu_events_map[0];
             tables->arch;
             tables++) {
                int ret = pmu_metrics_table__for_each_metric(&tables->metric_table, fn, data);

                if (ret)
                        return ret;
        }
        return 0;
}

const struct pmu_events_table *find_sys_events_table(const char *name)
{
        for (const struct pmu_sys_events *tables = &pmu_sys_event_tables[0];
             tables->name;
             tables++) {
                if (!strcmp(tables->name, name))
                        return &tables->event_table;
        }
        return NULL;
}

int pmu_for_each_sys_event(pmu_event_iter_fn fn, void *data)
{
        for (const struct pmu_sys_events *tables = &pmu_sys_event_tables[0];
             tables->name;
             tables++) {
                int ret = pmu_events_table__for_each_event(&tables->event_table,
                                                           /*pmu=*/ NULL, fn, data);

                if (ret)
                        return ret;
        }
        return 0;
}

int pmu_for_each_sys_metric(pmu_metric_iter_fn fn, void *data)
{
        for (const struct pmu_sys_events *tables = &pmu_sys_event_tables[0];
             tables->name;
             tables++) {
                int ret = pmu_metrics_table__for_each_metric(&tables->metric_table, fn, data);

                if (ret)
                        return ret;
        }
        return 0;
}

static const int metricgroups[][2] = {

};

const char *describe_metricgroup(const char *group)
{
        int low = 0, high = (int)ARRAY_SIZE(metricgroups) - 1;

        while (low <= high) {
                int mid = (low + high) / 2;
                const char *mgroup = &big_c_string[metricgroups[mid][0]];
                int cmp = strcmp(mgroup, group);

                if (cmp == 0) {
                        return &big_c_string[metricgroups[mid][1]];
                } else if (cmp < 0) {
                        low = mid + 1;
                } else {
                        high = mid - 1;
                }
        }
        return NULL;
}
